@@ -115,23 +115,47 @@ def BestNEnsemble(
                 for x in upper_forecasts.values()
             ]
         )
+        # checks only upper and middle, assuming lower follows others in NaN
+        # because the nanmedian and nanquantile are much slower than non na version
+        nan_flag = np.isnan(np.min(forecast_array)) or np.isnan(
+            np.min(u_forecast_array)
+        )
         if point_method == "midhinge":
-            ens_df = (
-                np.nanquantile(forecast_array, q=0.25, axis=0)
-                + np.quantile(forecast_array, q=0.75, axis=0)
-            ) / 2
-            ens_df_lower = (
-                np.nanquantile(l_forecast_array, q=0.25, axis=0)
-                + np.quantile(l_forecast_array, q=0.75, axis=0)
-            ) / 2
-            ens_df_upper = (
-                np.nanquantile(u_forecast_array, q=0.25, axis=0)
-                + np.quantile(u_forecast_array, q=0.75, axis=0)
-            ) / 2
+            if nan_flag:
+                ens_df = (
+                    np.nanquantile(forecast_array, q=0.25, axis=0)
+                    + np.nanquantile(forecast_array, q=0.75, axis=0)
+                ) / 2
+                ens_df_lower = (
+                    np.nanquantile(l_forecast_array, q=0.25, axis=0)
+                    + np.nanquantile(l_forecast_array, q=0.75, axis=0)
+                ) / 2
+                ens_df_upper = (
+                    np.nanquantile(u_forecast_array, q=0.25, axis=0)
+                    + np.nanquantile(u_forecast_array, q=0.75, axis=0)
+                ) / 2
+            else:
+                ens_df = (
+                    np.quantile(forecast_array, q=0.25, axis=0)
+                    + np.quantile(forecast_array, q=0.75, axis=0)
+                ) / 2
+                ens_df_lower = (
+                    np.quantile(l_forecast_array, q=0.25, axis=0)
+                    + np.quantile(l_forecast_array, q=0.75, axis=0)
+                ) / 2
+                ens_df_upper = (
+                    np.quantile(u_forecast_array, q=0.25, axis=0)
+                    + np.quantile(u_forecast_array, q=0.75, axis=0)
+                ) / 2
         else:
-            ens_df = np.nanmedian(forecast_array, axis=0)
-            ens_df_lower = np.nanmedian(l_forecast_array, axis=0)
-            ens_df_upper = np.nanmedian(u_forecast_array, axis=0)
+            if nan_flag:
+                ens_df = np.nanmedian(forecast_array, axis=0)
+                ens_df_lower = np.nanmedian(l_forecast_array, axis=0)
+                ens_df_upper = np.nanmedian(u_forecast_array, axis=0)
+            else:
+                ens_df = np.median(forecast_array, axis=0)
+                ens_df_lower = np.median(l_forecast_array, axis=0)
+                ens_df_upper = np.median(u_forecast_array, axis=0)
 
         ens_df = pd.DataFrame(ens_df, index=indices, columns=columnz)
         ens_df_lower = pd.DataFrame(ens_df_lower, index=indices, columns=columnz)
@@ -300,16 +324,17 @@ def mosaic_classifier(df_train, known):
             index=None if len(p_full) > 1 else [0],
         )
         upload = pd.concat([upload, missing_rows])
-    X = fill_median(
-        (summarize_series(df_train).transpose()).merge(
-            upload, left_index=True, right_on="series_id"
-        )
+    X = (
+        summarize_series(df_train)
+        .transpose()
+        .merge(upload, left_index=True, right_on="series_id")
     )
     X.set_index("series_id", inplace=True)  # .drop(columns=['series_id'], inplace=True)
     to_predict = X[X['model_id'].isna()].drop(columns=['model_id'])
     X = X[~X['model_id'].isna()]
     Y = X['model_id']
     Xf = X.drop(columns=['model_id'])
+    Xf = fill_median(Xf)
     # from sklearn.linear_model import RidgeClassifier
     # from sklearn.naive_bayes import GaussianNB
     from sklearn.ensemble import RandomForestClassifier
@@ -1217,6 +1242,7 @@ def generate_mosaic_template(
     full_mae_errors,
     smoothing_window=None,
     metric_name="MAE",
+    models_to_use=None,
     **kwargs,
 ):
     """Generate an ensemble template from results."""
@@ -1230,7 +1256,8 @@ def generate_mosaic_template(
     # remove slow models... tbd
     # select only models run through all validations
     run_count = local_results[['Model', 'ID']].groupby("ID").count()
-    models_to_use = run_count[run_count['Model'] == total_vals].index.tolist()
+    if models_to_use is None:
+        models_to_use = run_count[run_count['Model'] == total_vals].index.tolist()
     # begin figuring out which are the min models for each point
     id_array = np.array([y for y in sorted(full_mae_ids) if y in models_to_use])
     errors_array = np.array(
@@ -1379,6 +1406,7 @@ def MosaicEnsemble(
     fore, u_fore, l_fore = [], [], []
     row = (0, "Unknown", "Unknown", "Unknown")
     try:
+        # maybe this could be sped up by something like numpy take
         for row in melted.itertuples():
             fore.append(forecasts[row[3]][row[2]].iloc[row[1]])
             u_fore.append(upper_forecasts[row[3]][row[2]].iloc[row[1]])
