@@ -1,6 +1,6 @@
 """Lists of models grouped by aspects."""
 all_models = [
-    'ZeroesNaive',
+    'ConstantNaive',
     'LastValueNaive',
     'AverageValueNaive',
     'GLS',
@@ -34,7 +34,7 @@ all_models = [
     'DynamicFactorMQ',
 ]
 default = [
-    'ZeroesNaive',
+    'ConstantNaive',
     'LastValueNaive',
     'AverageValueNaive',
     'GLS',
@@ -43,7 +43,7 @@ default = [
     'ETS',
     'FBProphet',
     # 'RollingRegression',  # maybe not?
-    'GluonTS',  # downweight if that becomes an option
+    # 'GluonTS',  # downweight if that becomes an option
     'UnobservedComponents',
     'VAR',
     'VECM',
@@ -59,7 +59,7 @@ default = [
     'ARDL',
     # 'DynamicFactorMQ',
 ]
-colin = [
+best = [
     'LastValueNaive',
     'AverageValueNaive',
     'GLS',
@@ -88,7 +88,7 @@ colin = [
 ]
 # fastest models at any scale
 superfast = [
-    'ZeroesNaive',
+    'ConstantNaive',
     'LastValueNaive',
     'AverageValueNaive',
     'GLS',
@@ -96,7 +96,7 @@ superfast = [
 ]
 # relatively fast
 fast = [
-    'ZeroesNaive',
+    'ConstantNaive',
     'LastValueNaive',
     'AverageValueNaive',
     'GLS',
@@ -175,7 +175,7 @@ multivariate = [
 ]
 univariate = list((set(all_models) - set(multivariate)) - set(experimental))
 # USED IN AUTO_MODEL, models with no parameters
-no_params = ['ZeroesNaive', 'LastValueNaive', 'GLS']
+no_params = ['LastValueNaive', 'GLS']
 # USED IN AUTO_MODEL, ONLY MODELS WHICH CAN ACCEPT RANDOM MIXING OF PARAMS
 recombination_approved = [
     'SeasonalNaive',
@@ -207,7 +207,7 @@ recombination_approved = [
 ]
 # USED IN AUTO_MODEL for models that don't share information among series
 no_shared = [
-    'ZeroesNaive',
+    'ConstantNaive',
     'LastValueNaive',
     'AverageValueNaive',
     'GLM',
@@ -263,7 +263,7 @@ model_lists = {
     "slow": slow,
     "gpu": gpu,
     "regressor": regressor,
-    "colin": colin,
+    "best": best,
 }
 
 
